@@ -2,6 +2,7 @@
     Copyright (C) 2010  David Edmundson    <kde@davidedmundson.co.uk>
     Copyright (C) 2011  Dominik Schmidt    <dev@dominik-schmidt.de>
     Copyright (C) 2011  Francesco Nwokeka  <francesco.nwokeka@gmail.com>
+    Copyright (C) 2014  Marcin Ziemiński   <zieminn@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,10 +23,12 @@
 #define CHATWINDOW_H
 
 #include "chat-widget.h"
+#include "proxy-service.h"
 
 #include <KXmlGuiWindow>
 #include <KTabWidget>
 #include <KAction>
+#include <KActionMenu>
 
 namespace Sonnet {
     class DictionaryComboBox;
@@ -34,6 +37,7 @@ namespace Sonnet {
 class KIcon;
 class ChatTab;
 class QLabel;
+class QDBusPendingCallWatcher;
 
 class ChatWindow : public KXmlGuiWindow
 {
@@ -58,6 +62,7 @@ public:
      */
     ChatTab* getTab(const Tp::AccountPtr &account, const Tp::TextChannelPtr &incomingTextChannel);
     ChatTab* getCurrentTab();
+    QList<ChatTab*> tabs() const;
 
     void focusChat(ChatTab *tab);
 
@@ -83,12 +88,15 @@ private Q_SLOTS:
     void onAudioCallTriggered();                                /** start an audio call */
     void onBlockContactTriggered();                             /** Blocks contact */
     void onCurrentIndexChanged(int index);
+    void onTabMiddleClicked(int index);
     void onEnableSearchActions(bool enable);                    /** enables/disables menu search actions */
     void onFileTransferTriggered();                             /** start a file transfer (to be used only for 1on1 chats!) */
     void onFindNextText();                                      /** go to next text the user is searching for */
     void onFindPreviousText();                                  /** go to previous text the user is searching for */
     void onGenericOperationFinished(Tp::PendingOperation *op);
+    void onGetCurrentKeyboardLayoutFinished(QDBusPendingCallWatcher *watcher);
     void onInviteToChatTriggered();                             /** invite contact(s) to chat */
+    void onKeyboardLayoutChange(const QString& keyboardLayout);
     void onNextTabActionTriggered();                            /** go to next tab in the tabwidget */
     void onPreviousTabActionTriggered();                        /** go to previous tab in the tabwidget */
     void onSearchActionToggled();                               /** toggle search bar visibility */
@@ -98,7 +106,9 @@ private Q_SLOTS:
     void onVideoCallTriggered();                                /** start a video call */
     void onUnblockContactTriggered();                           /** Unblocks contact when already blocked */
     void onShareDesktopTriggered();                             /** start a desktop share */
+    void onShowInfoTriggered();                                 /** show contact info */
     void onOpenLogTriggered();                                  /** Starts ktp-log-viewer accountId contactId */
+    void onOpenContactListTriggered();                          /** Opens contact list */
     void onClearViewTriggered();                                /** Clears current view */
     void setTabSpellDictionary(const QString &dict);            /** set the spelling language for the current chat tab*/
     void toggleBlockButton(bool contactIsBlocked);              /** Toggle block/unblock action according to the flag */
@@ -110,6 +120,14 @@ private Q_SLOTS:
     void updateSendMessageShortcuts();
     void onReloadTheme();
     void onCollaborateDocumentTriggered();
+    void onLeaveChannelTriggered();
+    /** otr related handlers */
+    void onOtrStatusChanged(OtrStatus status);
+    void onStartRestartOtrTriggered();
+    void onStopOtrTriggered();
+    void onAuthenticateBuddyTriggered();
+    void onKeyGenerationStarted(Tp::AccountPtr);
+    void onKeyGenerationFinished(Tp::AccountPtr, bool error);
 
 protected Q_SLOTS:
     void showSettingsDialog();
@@ -135,6 +153,9 @@ private:
     /** creates and adds custom actions for the chat window */
     void setupCustomActions();
 
+    /** sets up otr actions */
+    void setupOTR();
+
     /** setters for chat actions */
     void setAudioCallEnabled(bool enable);
     void setBlockEnabled(bool enable);
@@ -144,6 +165,7 @@ private:
     void setShareDesktopEnabled(bool enable);
     void setPreviousConversationsEnabled(bool enable);
     void setCollaborateDocumentEnabled(bool enable);
+    void setShowInfoEnabled(bool enable);
 
     /** starts audio call with given contact
      * @param account account sending the audio call request
@@ -186,13 +208,22 @@ private:
      */
     void offerDocumentToChatroom(const Tp::AccountPtr& account, const QString& roomName);
 
+    /**
+     * @brief Restores the System's keyboard layout to the layout of the current ChatTab
+     * @param chatTab The tab that the language is going to be restored from
+     */
+    void restoreKeyboardLayout(ChatTab *chatTab);
+
     KAction *m_sendMessage;
 
     KTabWidget *m_tabWidget;
 
+    QDBusInterface *m_keyboardLayoutInterface;
     Sonnet::DictionaryComboBox *m_spellDictCombo;
     QLabel *m_accountIconLabel;
     qreal m_zoomFactor;
+    KActionMenu *m_otrActionMenu;
+    ProxyServicePtr m_proxyService;
 };
 
 #endif // CHATWINDOW_H
