@@ -1,6 +1,7 @@
 /*
-    Copyright (C) 2010  David Edmundson <kde@davidedmundson.co.uk>
-    Copyright (C) 2011  Dominik Schmidt <dev@dominik-schmidt.de>
+    Copyright (C) 2010  David Edmundson    <kde@davidedmundson.co.uk>
+    Copyright (C) 2011  Dominik Schmidt    <dev@dominik-schmidt.de>
+    Copyright (C) 2014  Marcin Ziemiński   <zieminn@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +33,7 @@
 #include <TelepathyQt/AccountManager>
 
 #include <KTp/core.h>
+#include <KTp/OTR/types.h>
 
 #include <KAboutData>
 #include <KCmdLineArgs>
@@ -39,11 +41,12 @@
 int main(int argc, char *argv[])
 {
     KAboutData aboutData("ktp-text-ui", 0,
-                         ki18n("Telepathy Text Ui"),
+                         ki18n("Chat Application"),
                          KTP_TEXT_UI_VERSION);
     aboutData.addAuthor(ki18n("David Edmundson"), ki18n("Developer"), "david@davidedmundson.co.uk");
     aboutData.addAuthor(ki18n("Dominik Schmidt"), ki18n("Developer"), "kde@dominik-schmidt.de");
     aboutData.addAuthor(ki18n("Francesco Nwokeka"), ki18n("Developer"), "francesco.nwokeka@gmail.com");
+    aboutData.addAuthor(ki18n("Marcin Ziemiński"), ki18n("Developer"), "zieminn@gmail.com");
     aboutData.setProductName("telepathy/text-ui"); //set the correct name for bug reporting
     aboutData.setLicense(KAboutData::License_GPL_V2);
     aboutData.setProgramIconName(QLatin1String("telepathy-kde"));
@@ -51,6 +54,7 @@ int main(int argc, char *argv[])
     KCmdLineArgs::init(argc, argv, &aboutData);
 
     Tp::registerTypes();
+    KTp::registerOtrTypes();
 
     Tp::ChannelFactoryPtr channelFactory = Tp::ChannelFactory::create(QDBusConnection::sessionBus());
     channelFactory->addCommonFeatures(Tp::Channel::FeatureCore);
